/**   @file  oom.h
 *   @brief
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  21/06/2024
 *      Revision:  none
 * Last modified:  2024-07-26 00:57
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __OOM_H__
#define __OOM_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef RAMLIMIT_HARD
#define RAMLIMIT_HARD (7*1024*1024)
#endif
#ifndef RAMLIMIT_SOFT
#define RAMLIMIT_SOFT (5*1024*1024)
#endif

#include <stddef.h>                             /* size_t */

/*     void myfunction() {
 *         void* ptr;
 *         ptr = NULL;
 *         // Keep at most 100 bytes available
 *         OOM_BEGIN(0.1);
 *         ptr=malloc(2000);
 *         OOM_END;
 *         if (NULL == ptr)
 *             fprintf(stderr,"malloc failed\n");
 *         else
 *             free(ptr);
 *     }
 *
 *     size_t ramlimit;
 *     ramlimit=64*1024*1024;
 *     printf ("Limit memory usage to %uMB : ",ramlimit/1024/1024);
 *     ramlimit = oomtest_setup(ramlimit);
 *     printf ("%uB (%uMB)\n",ramlimit,ramlimit/1024/1024);
 *
 *     // not constrained (only by the 64MB ramlimit)
 *     myfunction();
 *
 *     // OOM activation to eat all memory
 *     oomtest_enable(64*1024*1024);
 *     myfunction();
 */

/** Sets the oomtest helpers hard rlimit and enables the oomtest helper features
 *
 * This function needs to be invoqued BEFORE any other helper from the
 * framework. It will configure an hard RAM limit for the current process and
 * each of his children. Then, it will enable the other oomtest helper
 * functions.
 *
 * It refuses to set a limit over the actually installed physical RAM to limit
 * pushing RAM pages to the swap. If the requested value is zero, it will
 * default to the actually installed physical RAM.
 *
 * If anything goes wrong or did not behaves as expected, it abort the process
 * to avoid RAM bombing and swapping.
 *
 * Despite it was designed to be invoked only once, from the main parent
 * process, it can be invoked several times as long as the hardlimit parameter
 * is always less than the previous call, otherwise it will fail and abort the
 * process.
 *
 * \param [in] hardlimit the size in bytes to limit the process to.
 *
 * \return Returns the actual configured size. It should be the same
 * value as the hardlimit parameter, otherwise something went wrong, it was not
 * detected and the process was not aborted (which is a bug to report)
 *
 * \sa oomtest_enable
 * \sa oomtest_disable
 * \sa oomtest_fill
 * \sa oomtest_free
 * */
size_t oomtest_config(const size_t hardlimit);

/** Starts a new oomtest environment or reconfigure the soft limit
 *
 * This function can only be invoked after at least a first call to
 * oomtest_config to initialize the oomtest helpers environment.  If invocated
 * before, it will abort the current process to help detecting mistakes in the
 * test code.
 *
 * It will (soft)limit the current process and his children to the provided
 * value in bytes. Then, it will enable the fill/free oomtest helper functions
 * which are disabled otherwise.
 *
 * It is designed to be invoked at the begining of a test case to apply for all
 * tests in this testcase.
 *
 * In case of any unexpected behavior, it should abort the current process.
 *
 * \param [in] softlimit Soft limit to set in bytes. It has to be less than the
 * hardlimit otherwise it will fail and abort. If set to 0, it will apply the
 * same value as the hardlimit.
 *
 * \return Returns the actually set value as a softlimit, either the requested
 * value or the hardlimit in case of 0 requested.
 *
 * \sa oomtest_enable
 * \sa oomtest_disable
 * \sa oomtest_fill
 * \sa oomtest_free
 * */
extern size_t (*oomtest_enable)(const size_t softlimit);

/** Stops the current oomtest
 *
 * This function can be invoked any time after the oomtest_config initialized
 * the environment. If invocated before, it will abort the current process to
 * help detecting mistakes in the test code.
 *
 * This function disables the fill/free oomtest functions and free the allocated
 * RAM before reverting the soft limit to the hard limit value.
 *
 * It is designed to be invoked t the end of a test case.
 *
 * Despite it could be invoked twice or without a prior call to
 * oomtest_enable, it is illegal and not allowed to help detecting
 * mistakes in the test code.
 *
 * \return Returns the applied soft limit, which should be the same as the hard
 * limit.
 *
 * \sa oomtest_enable
 * \sa oomtest_disable
 * \sa oomtest_fill
 * \sa oomtest_free
 * */
extern size_t (*oomtest_disable)();

/** Starts an almost OOM single and simple test
 *
 * This function is only enabled after a call to oomtest_enable and will have no
 * effect otherwise. Its goal is to completely fill the RAM until the very last
 * bytes, to keep only between minHeap and maxHeap bytes available in the heap
 * and minStack bytes in the stack.
 *
 * If oomtest_enable was not invoked beforehand, this function will simply
 * return without any RAM consumption.
 *
 * It should be called immediately before the test and should be reverted with
 * oomtest_free immediately after. It can create so much pressure on the
 * available memory that a simple printf could fail.
 *
 * It is designed to fail if invoked twice as this is very probably a mistake in
 * the test code. Should you need to change the RAM filling values, first call
 * oomtest_free and reapply oomtest_fill. This ensure a really wanted behavior
 * and not a mistake in your test code.
 *
 * \param [in] minHeap
 * \param [in] minStack
 *
 * \return Returns the allocated size to fill the memory
 *
 * \sa oomtest_enable
 * \sa oomtest_disable
 * \sa oomtest_fill
 * \sa oomtest_free
 * */
extern size_t (*oomtest_fill)(const size_t minHeap, const size_t minStack);

/** Ends a single simple OOM test
 *
 * If this function is invoked between an oomtest_enable and an oomtest_disable
 * invocations, it deallocates the RAM allocated by the oomtest_fill function. It
 * should be called immediately after the single and simple test because the RAM
 * pressure can even make a printf to fail.
 *
 * If called without a prior oomtest_enable invocation, this function simply
 * returns without any action.
 *
 * This function is designed to fail and abort the process if invocated whereas
 * there is no current RAM allocated, when called twice, for example. This helps
 * to avoid mistakes in the test scenario.
 *
 * */
extern void (*oomtest_free)();

extern unsigned char oomtest_enabled();

#endif  /*__OOM_H__ */
